# Classes for each type

class LoadableRecord

  def initialize
  end
  def LoadableRecord.create(row)
    return nil
  end
  def height
    (@y2.to_f-@y1.to_f).abs
  end
  def width
    (@x2.to_f-@x1.to_f).abs
  end
end

class WorldMapArea < LoadableRecord
  attr_accessor :wma_id, :map_id, :area_id, :name, :y1, :y2, :x1, :x2, :virtual_id, :dungeon_map_id, :relate_map_id, :map_floors, :map_type, :instance_continent
  def self.create(row)
    wma = WorldMapArea.new
    wma.wma_id = row[0]
    wma.map_id = row[1]
    wma.area_id = row[2]
    wma.name = row[3]
    wma.y1 = row[6]
    wma.y2 = row[7]
    wma.x1 = -row[4]
    wma.x2 = -row[5]
    wma.virtual_id = row[8]
    wma.dungeon_map_id = row[9]
    wma.relate_map_id = row[10]
    wma.map_floors = []
    wma.map_type = 0
    wma.instance_continent = 0
    return wma
  end
  def <=>(other)
    return wma_id <=> other.wma_id
  end
  
  def has_continent?
    return @map_id == 0 || @map_id == 1 || @map_id == 530 || @map_id == 571 || @map_id == -1 || @map_id == 870
  end
  
  def instance_continent
    if @map_type < 1 || @map_type > 2
      return 0
    end
    
    if @instance_continent == 0
      return 1
    elsif @instance_continent == 1
      return 2
    elsif @instance_continent == 530
      return 3
    elsif @instance_continent == 571
      return 4
    elsif @instance_continent == -1
      return 5
    elsif @instance_continent == 870
      return 6
    else
      return 0
    end    
  end
  
  def continent
    # Fixes for BC zones that are marked as outlands, but located in real world
    if @wma_id == 499 || @wma_id == 480 || @wma_id == 462 || @wma_id == 463
      return 2
    end
    if @wma_id == 464 || @wma_id == 471 || @wma_id == 476
      return 1
    end
    
    if @map_id == 0
      return 1
    elsif @map_id == 1
      return 2
    elsif @map_id == 530
      return 3
    elsif @map_id == 571
      return 4
    elsif @map_id == -1
      return 5
    elsif @map_id == 870
      return 6
    else
      return 0
    end
  end
end

class WorldMapContinent < LoadableRecord
  attr_accessor :wmc_id, :map_id, :y1, :y2, :x1, :x2, :u1, :u2
  def self.create(row)
    wmc = WorldMapContinent.new
    wmc.wmc_id= row[0]
    wmc.map_id= row[1]
    wmc.x2= row[11]
    wmc.y2= row[12]
    wmc.x1= row[9] 
    wmc.y1= row[10]
    wmc.u1 = row[6]
    wmc.u2 = row[7]
    return wmc
  end
end

class DungeonMap < LoadableRecord
  include Comparable
  attr_accessor :sub_id, :wma_id, :floor, :y1, :y2, :x1, :x2, :wmc_id
  def self.create(row)
    dm = DungeonMap.new
    dm.sub_id = row[0]
    dm.wma_id = row[1]
    dm.floor = row[2]
    dm.y1= row[4]
    dm.y2= row[6]
    dm.x1= -row[3] 
    dm.x2= -row[5]
    dm.wmc_id=row[7]
    return dm
  end
  
  def duplicate(other)
    @sub_id = other.sub_id
    @wma_id = other.wma_id
    @floor = other.floor
    @y1 = other.y1
    @y2 = other.y2
    @x1 = other.x1
    @v2 = other.x2
    @wmc_id = other.wmc_id
  end
  
  def is_continent?
    return @wma_id == 0 || @wma_id == 1 || @wma_id == 530 || @wma_id == 571 || @wma_id == -1 || @wma_id == 870
  end
  def ==(another)
    if another.instance_of?(DungeonMap)
      return @floor == another.floor && width.to_i == another.width.to_i && height.to_i == another.height.to_i
    else
      return false
    end
  end
  def <=>(another)
    return @floor <=> another.floor
  end
  def eql?(another)
    if another.instance_of?(DungeonMap)
      return @floor == another.floor && width.to_i == another.width.to_i && height.to_i == another.height.to_i
    else
      return false
    end
  end
  def hash
    return [@floor, width.to_i, height.to_i].hash
  end
end

class Map < LoadableRecord
  MAP_COMMON = 0
  MAP_INSTANCE = 1
  MAP_RAID = 2
  MAP_BG = 3
  MAP_ARENA = 4
  MAP_SENARIO = 5
  attr_accessor :map_id, :internal_name, :map_type, :map_flags, :pvp, :name, :linked_zone,:entrace_map_id, :entrance_x, :entrance_y, :expansion, :max_players, :phase_map_id
  def self.create(row)
    mp = Map.new
    mp.map_id= row[0]
    mp.internal_name= row[1]
    mp.map_type= row[2]
    mp.map_flags= row[3]
    mp.pvp= row[5]
    mp.name= row[6]
    mp.linked_zone= row[7]
    mp.entrace_map_id= row[12]
    mp.entrance_x= row[13]
    mp.entrance_y= row[14]
    mp.expansion= row[16]
    mp.max_players= row[18]
    mp.phase_map_id = row[19]
    return mp
  end
  def is_instance?
    return @map_type == MAP_INSTANCE || @map_type == MAP_RAID
  end
  def is_expansion?
    return @expansion > 0
  end
  def is_continent?
    # Kalimdor, Eastern Kingdoms, Outlands, Northrend, Maelstrom, Pandaria
    return @map_id == 0 || @map_id == 1 || @map_id == 530 || @map_id == 571 || @map_id == -1 || @map_id == 870
  end
end

class WorldMapTransforms < LoadableRecord
  attr_accessor :map_id, :src_map, :sx1, :sy1, :sx2, :sy2, :dst_map, :xoff, :yoff, :uk1, :uk2
  def self.create(row)
    wt = WorldMapTransforms.new
    wt.map_id = row[0]
    wt.src_map = row[1]
    wt.sx1 = row[2]
    wt.sy1 = row[3]
    wt.sx2 = row[4]
    wt.sy2 = row[5]
    wt.dst_map = row[6]
    wt.xoff = row[7]
    wt.yoff = row[8]
    wt.uk1 = row[9]
    wt.uk2 = row[10]
    return wt
  end
end